import { POST_PATH, POST_ROUTE_PREFIX } from '@blog/config';
import { readdir } from 'node:fs/promises';
import type { Post } from '@blog/models/Post';
import * as yamlFront from 'yaml-front-matter';
import { marked } from 'marked';

export async function readPostFile(fileName: string): Promise<Post> {
  const file = Bun.file(POST_PATH + '/' + fileName);
  const fileContent = await file.text();
  const parsedData = yamlFront.loadFront(fileContent);  
  const postHtml = await marked.parse(parsedData.__content);

  return {
    meta: {
      title: parsedData.title,
      description: parsedData.description,
      date: new Date(parsedData.date),
      draft: parsedData?.draft || false,
      tags: parsedData.tags,
      slug: POST_ROUTE_PREFIX + '/' + fileName,
    },
    content: parsedData.__content,
    html: postHtml
  };
}

export class PostFileService {
  private posts: Map<string, Post>; 

  public constructor(posts: Map<string, Post>) {
    this.posts = posts;
  }

  public static async create() {
    const posts = new Map<string, Post>();
    const postFiles: string[] = await readdir(POST_PATH);

    for (const postFile in postFiles) {
      const post = await readPostFile(postFile);
      const key = post.meta?.slug || postFile.slice(0, -3);
      posts.set(key, post);
    }

    return new PostFileService(posts);
  }

  public getPostHtml(slug: string): string {
    const html = this.posts.get(slug)?.html;

    if (!html?.length) {
      throw new Error("Post does not exist.");
    }

    return html;
  }

  public getPostRawContent(slug: string): string {
    const content = this.posts.get(slug)?.content;

    if (!content?.length) {
      throw new Error("Post does not exist.");
    }

    return content;
  }
}
